@@ -1,26 +1,7 @@
-#'
-#'
-#' Calculates the FIML objective function
-#' @param ImpCov expected covariance matrix.
-#' @param data required dataset.
-#' @param Areg A matrix with current parameter estimates.
-#' @param lambda penalty value.
-#' @param alpha mixture for elastic net.
-#' @param type penalty type.
-#' @param pen_vec vector of penalized parameters.
-#' @param nvar number of variables.
-#' @keywords fit maximum likelihood regularization
-#'
-#' @examples
-#' \dontrun{
-#' fiml_calc()
-#' }
-
-
 
 fiml_calc = function(ImpCov,data,Areg,lambda,alpha,type,pen_vec,nvar){
 
-
+# look at lav_objective.R from lavaan
   m = dim(ImpCov)[1]
   IntCol = which(colnames(Areg) == "1")
   IntCol2 = colnames(Areg[,1:nvar])
@@ -46,7 +27,7 @@ fiml_calc = function(ImpCov,data,Areg,lambda,alpha,type,pen_vec,nvar){
         sub1 = as.numeric(cbind(person1,0) - meanvec)
         indFit = K - log(det(ImpCov))  + t(sub1) %*% solve(ImpCov) %*% sub1
       }
-      fit = fit + indFit
+      fit = fit + log(indFit)
     }
 
 
@@ -55,6 +36,6 @@ fiml_calc = function(ImpCov,data,Areg,lambda,alpha,type,pen_vec,nvar){
   }
 
 
-  #-2*fit
-  -2 * fit
+  fit
+
 }
